@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2021 Free Software Foundation, Inc.
 #
 #  GNU Mailutils is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -27,10 +27,10 @@ registrar.register_format ()
 mbox = mailbox.MailboxDefault (args)
 mbox.open ()
 
-print "Total: %d" % len (mbox)
+print("Total: %d" % len (mbox))
 
 for msg in mbox:
-  print "%s %s" % (msg.header[MU_HEADER_FROM],
-                   msg.header.get_value (MU_HEADER_SUBJECT, "(NO SUBJECT)"))
+  print("%s %s" % (msg.header[MU_HEADER_FROM],
+                   msg.header.get_value (MU_HEADER_SUBJECT, "(NO SUBJECT)")))
 
 mbox.close ()
