@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,6 +28,7 @@
 #include <mailutils/errno.h>
 #include <mailutils/stdstream.h>
 #include <mailutils/stream.h>
+#include <mailutils/locus.h>
 
 void
 mu_diag_init ()
@@ -56,17 +56,62 @@ mu_diag_output (int level, const char *fmt, ...)
 }
 
 void
-mu_diag_at_locus (int level, struct mu_locus const *loc, const char *fmt, ...)
+mu_vdiag_at_locus_range (int level, struct mu_locus_range const *loc,
+			 const char *fmt, va_list ap)
+{
+  struct mu_locus_range old_loc = MU_LOCUS_RANGE_INITIALIZER;
+  int old_mode;
+  int restore = 0;
+  
+  if (loc)
+    {
+      if (mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+			   MU_IOCTL_LOGSTREAM_GET_LOCUS_RANGE, &old_loc) == 0)
+	{
+	  if (mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+			       MU_IOCTL_LOGSTREAM_GET_MODE, &old_mode) == 0)
+	    {
+	      int mode = old_mode | MU_LOGMODE_LOCUS;
+	      mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+			       MU_IOCTL_LOGSTREAM_SET_MODE, &mode);
+	      mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+			       MU_IOCTL_LOGSTREAM_SET_LOCUS_RANGE, (void*) loc);
+	      restore = 1;
+	    }
+	}
+    }
+
+  mu_diag_voutput (level, fmt, ap);
+
+  if (restore)
+    {
+      mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+		       MU_IOCTL_LOGSTREAM_SET_LOCUS_RANGE, &old_loc);
+      mu_stream_ioctl (mu_strerr, MU_IOCTL_LOGSTREAM,
+		       MU_IOCTL_LOGSTREAM_SET_MODE, &old_mode);
+      mu_locus_range_deinit (&old_loc);
+    }
+}
+
+void
+mu_diag_at_locus_range (int level, struct mu_locus_range const *loc,
+			const char *fmt, ...)
 {
   va_list ap;
-
   va_start (ap, fmt);
-  if (loc && loc->mu_file)
-    mu_stream_printf (mu_strerr, "\033f<%d>%s\033l<%u>\033c<%u>",
-		      (unsigned) strlen (loc->mu_file), loc->mu_file,
-		      loc->mu_line,
-		      loc->mu_col);
-  mu_diag_voutput (level, fmt, ap);
+  mu_vdiag_at_locus_range (level, loc, fmt, ap);
+  va_end (ap);
+}
+
+void
+mu_diag_at_locus_point (int level, struct mu_locus_point const *loc,
+			const char *fmt, ...)
+{
+  va_list ap;
+  struct mu_locus_range lr = MU_LOCUS_RANGE_INITIALIZER;
+  lr.beg = *loc;
+  va_start (ap, fmt);
+  mu_vdiag_at_locus_range (level, &lr, fmt, ap);
   va_end (ap);
 }
 
@@ -74,7 +119,7 @@ void
 mu_diag_vprintf (int level, const char *fmt, va_list ap)
 {
   mu_diag_init ();
-  mu_stream_printf (mu_strerr, "\033<%d>", level);
+  mu_stream_printf (mu_strerr, "\033s<%d>", level);
   mu_stream_vprintf (mu_strerr, fmt, ap);
 }
 
@@ -142,10 +187,10 @@ mu_diag_funcall (mu_log_level_t level, const char *func,
   if (err)
     /* TRANSLATORS: First %s stands for function name, second for its
        arguments, third one for the actual error message. */
-    mu_diag_output (level, _("%s(%s) failed: %s"), func, arg ? arg : "",
+    mu_diag_output (level, _("%s(%s) failed: %s"), func, mu_prstr (arg),
 		    mu_strerror (err));
   else
     /* TRANSLATORS: First %s stands for function name, second for its
        arguments. */
-    mu_diag_output (level, _("%s(%s) failed"), func, arg ? arg : "");
+    mu_diag_output (level, _("%s(%s) failed"), func, mu_prstr (arg));
 }
