@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003-2019 Free Software Foundation, Inc.
+   Copyright (C) 2003-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library.  If not, see 
+   Public License along with this library.  If not, see
    <http://www.gnu.org/licenses/>. */
 
 /* This file implements a DOT filter, useful for data I/O in
@@ -20,7 +20,7 @@
    "byte-stuffs" the input by outputting an additional '.' in front
    of any '.' appearing at the beginning of a line.  Upon closing the
    filter in this mode, it outputs additional ".\n".
-   
+
    When decoding, the reverse is performed: any '.' appearing at the
    beginning of a line is removed.  A single dot on a line by itself
    marks end of the stream.
@@ -41,9 +41,8 @@
 
 enum dot_decode_state
   {
-    dot_decode_init,  /* initial state */
-    dot_decode_char,  /* Any character excepting [\r\n.] */
-    dot_decode_lf,    /* prev. char was \n */
+    dot_decode_bol,   /* beginning of line */
+    dot_decode_char,  /* Any character excepting [\n.] */
     dot_decode_dot,   /* 2 prev. chars were \n. */
     dot_decode_end   /* final state, a \n.\n seen. */
   };
@@ -53,32 +52,24 @@ new_decode_state (enum dot_decode_state state, int c)
 {
   switch (state)
     {
-    case dot_decode_init:
+    case dot_decode_bol:
       switch (c)
 	{
 	case '.':
 	  return dot_decode_dot;
+	case '\n':
+	  return dot_decode_bol;
 	}
       break;
-      
+
     case dot_decode_char:
       switch (c)
 	{
 	case '\n':
-	  return dot_decode_lf;
+	  return dot_decode_bol;
 	}
       break;
-      
-    case dot_decode_lf:
-      switch (c)
-	{
-	case '.':
-	  return dot_decode_dot;
-	case '\n':
-	  return dot_decode_lf;
-	}
-      break;
-      
+
     case dot_decode_dot:
       switch (c)
 	{
@@ -93,8 +84,7 @@ new_decode_state (enum dot_decode_state state, int c)
   return dot_decode_char;
 }
 
-/* Move min(isize,osize) bytes from iptr to optr, replacing each \r\n
-   with \n. */
+/* Move min(isize,osize) bytes from iptr to optr, unstuffing '..' sequences */
 static enum mu_filter_result
 _dot_decoder (void *xd,
 	      enum mu_filter_command cmd,
@@ -110,34 +100,36 @@ _dot_decoder (void *xd,
   switch (cmd)
     {
     case mu_filter_init:
-      *pstate = dot_decode_init;
+      *pstate = dot_decode_bol;
       return mu_filter_ok;
-      
+
     case mu_filter_done:
       return mu_filter_ok;
-      
+
     default:
       break;
     }
-  
+
   iptr = (const unsigned char *) iobuf->input;
   isize = iobuf->isize;
   optr = iobuf->output;
   osize = iobuf->osize;
 
-  for (i = j = 0; *pstate != dot_decode_end && i < isize && j < osize; i++)
+  for (i = j = 0; i < isize && j < osize; i++)
     {
       unsigned char c = *iptr++;
       int curstate = *pstate;
-      
+
       *pstate = new_decode_state (curstate, c);
-      if (!(c == '.' && (curstate == dot_decode_init ||
-			 curstate == dot_decode_lf)))
-	optr[j++] = c;
+      if (c == '.' && curstate == dot_decode_bol)
+	    continue;
+      if (*pstate == dot_decode_end)
+	{
+	  iobuf->eof = 1;
+	  break;
+	}
+      optr[j++] = c;
     }
-  
-  if (*pstate == dot_decode_end)
-    iobuf->eof = 1;
 
   iobuf->isize = i;
   iobuf->osize = j;
@@ -149,7 +141,7 @@ enum dot_encode_state
     dot_encode_init,  /* initial state */
     dot_encode_char,  /* Any character excepting \n */
     dot_encode_lf,    /* prev. char was \n */
-  };    
+  };
 
 static enum dot_encode_state
 new_encode_state (int c)
@@ -162,8 +154,8 @@ new_encode_state (int c)
   return dot_encode_char;
 }
 
-/* Move min(isize,osize) bytes from iptr to optr, replacing each \n
-   with \r\n.  Any input \r\n sequences remain untouched. */
+/* Move min(isize,osize) bytes from iptr to optr, byte-stuffing each
+   '.' appearing at the beginning of a line */
 static enum mu_filter_result
 _dot_encoder (void *xd,
 	      enum mu_filter_command cmd,
@@ -175,21 +167,21 @@ _dot_encoder (void *xd,
   size_t isize;
   char *optr;
   size_t osize;
-  int *state = xd;
-  
+  enum dot_encode_state *state = xd;
+
   switch (cmd)
     {
     case mu_filter_init:
       *state = dot_encode_init;
       return mu_filter_ok;
-      
+
     case mu_filter_done:
       return mu_filter_ok;
 
     default:
       break;
     }
-  
+
   iptr = (const unsigned char *) iobuf->input;
   isize = iobuf->isize;
   optr = iobuf->output;
@@ -203,7 +195,7 @@ _dot_encoder (void *xd,
       if (c == '.' && (curstate == dot_encode_init ||
 		       curstate == dot_encode_lf))
 	{
- 	  if (j + 2 > osize)
+	  if (j + 2 > osize)
 	    {
 	      if (i == 0)
 		{
@@ -226,11 +218,12 @@ _dot_encoder (void *xd,
     {
       switch (*state)
 	{
+	case dot_encode_init:
 	case dot_encode_lf:
 	  if (j + 2 > osize)
 	    result = mu_filter_again;
 	  break;
-	      
+
 	default:
 	  if (j + 3 > osize)
 	    result = mu_filter_again;
