@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ edit0 (msgset_t *mspec, mu_message_t msg, void *data)
   util_do_command ("shell %s %s", getenv ("EDITOR"), file);
   remove (file);
   free (file);
-  set_cursor (mspec->msg_part[0]);
+  set_cursor (msgset_msgno (mspec));
   return 0;
 }
 
