@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -149,7 +149,7 @@ hdr_text (struct header_call_args *args, void *data)
 static char *
 hdr_cur (struct header_call_args *args, void *data)
 {
-  if (is_current_message (args->mspec->msg_part[0]))
+  if (is_current_message (msgset_msgno (args->mspec)))
     return (char*) data;
   return " ";
 }
@@ -327,7 +327,7 @@ static char *
 hdr_number (struct header_call_args *args, void *data)
 {
   char buf[UINTMAX_STRSIZE_BOUND];
-  return header_buf_string (args, umaxtostr (args->mspec->msg_part[0], buf));
+  return header_buf_string (args, umaxtostr (msgset_msgno (args->mspec), buf));
 }
 
 /* %o */
