@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -166,7 +166,13 @@ mail_mbox_commit (void)
 
 	  status = mu_mailbox_append_message (dest_mbox, msg);
 	  if (status)
-	    mu_error (_("Cannot append message: %s"), mu_strerror (status));
+	    {
+	      mu_url_t url = NULL;
+	      mu_mailbox_get_url (dest_mbox, &url);
+	      mu_error (_("Cannot append message to %s: %s"),
+			util_url_to_string (url),
+			mu_strerror (status));
+	    }
 	  else
 	    {
 	      mu_attribute_set_deleted (attr);
