@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_file, file_compl },
   { "folders",	"folders",	"folders",			0,
     mail_folders, no_compl },
-  { "fo",	"followup",	"fo[llowup] [msglist]",		EF_SEND,
+  { "fo",	"followup",	"fo[llowup] [message]",		EF_SEND,
     mail_followup, msglist_compl },
   { "f",	"from",		"f[rom] [msglist]",		0,
     mail_from, msglist_compl },
@@ -84,6 +84,8 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_list, no_compl },
   { "m",	"mail",		"m[ail] [address...]",		EF_SEND,
     mail_send, alias_compl },
+  { "M",        "Mail",         "M[ail] [address...]",		EF_SEND,
+    mail_send, alias_compl },
   { "mb",	"mbox",		"mb[ox] [msglist]",		0,
     mail_mbox, msglist_compl },
   { "n",	"next",		"n[ext] [message]",		0,
@@ -108,9 +110,9 @@ static const struct mail_command_entry mail_command_table[] = {
     mail_reply, msglist_compl },
   { "R",	"Respond",	"R[espond] [msglist]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"reply",	"r[eply] [msglist]",		EF_SEND,
+  { "r",	"reply",	"r[eply] [message]",		EF_SEND,
     mail_reply, msglist_compl },
-  { "r",	"respond",	"r[espond] [msglist]",		EF_SEND,
+  { "r",	"respond",	"r[espond] [message]",		EF_SEND,
     mail_reply, msglist_compl },
   { "ret",	"retain",	"ret[ain] [header-field]",	0,
     mail_retain, no_compl },
