@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -129,9 +129,9 @@ main (int argc, char **argv)
       return 1;
     }
 
-  if ((rc = mu_temp_file_stream_create (&tmp, NULL, 0))) 
+  if ((rc = mu_temp_stream_create (&tmp, 0))) 
     {
-      mu_error (_("Cannot open temporary file: %s"),
+      mu_error (_("Cannot open temporary stream: %s"),
 		mu_strerror (rc));
       return 1;
     }
@@ -156,7 +156,7 @@ main (int argc, char **argv)
 	  mu_opool_t opool;
 	  const char *prompt = name;
 	  
-	  mu_opool_create (&opool, 1);
+	  mu_opool_create (&opool, MU_OPOOL_ENOMEMABRT);
 	  do
 	    {
 	      size_t len;
