@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@ _repr1 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME1 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyObject *
@@ -37,7 +37,7 @@ _repr2 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME2 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyObject *
@@ -45,7 +45,7 @@ _repr3 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME3 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyObject *
@@ -53,134 +53,41 @@ _repr4 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME4 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 
 static PyTypeObject PyAuthorityType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME1,  /* tp_name */
-  sizeof (PyAuthority),      /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr1,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr1,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME1,
+  .tp_basicsize = sizeof (PyAuthority),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr1,
+  .tp_str = _repr1,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 static PyTypeObject PyTicketType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME2,  /* tp_name */
-  sizeof (PyTicket),         /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr2,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr2,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME2,
+  .tp_basicsize = sizeof (PyTicket),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr2,
+  .tp_str = _repr2,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 static PyTypeObject PyWicketType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME3,  /* tp_name */
-  sizeof (PyWicket),         /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr3,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr3,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME3,
+  .tp_basicsize = sizeof (PyWicket),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr3,
+  .tp_str = _repr3,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 void
@@ -202,78 +109,48 @@ _getattr4 (PyObject *self, char *name)
     return NULL;
 
   if (strcmp (name, "name") == 0)
-    return PyString_FromString (ad->name);
+    return PyUnicode_FromString (ad->name);
   else if (strcmp (name, "passwd") == 0)
-    return PyString_FromString (ad->passwd);
+    return PyUnicode_FromString (ad->passwd);
 
-  /* FIXME: The use of PyInt_FromSize_t to convert uid_t and gid_t is
+  /* FIXME: The use of PyLong_FromSize_t to convert uid_t and gid_t is
      a bit dubious, but so far there's no other feasible way in Python,
-     save for converting uid (gid) to string and using PyInt_FromString. */
+     save for converting uid (gid) to string and using PyLong_FromString. */
   else if (strcmp (name, "uid") == 0)
-    return PyInt_FromSize_t (ad->uid);
+    return PyLong_FromSize_t (ad->uid);
   else if (strcmp (name, "gid") == 0)
-    return PyInt_FromSize_t (ad->gid);
+    return PyLong_FromSize_t (ad->gid);
   else if (strcmp (name, "gecos") == 0)
-    return PyString_FromString (ad->gecos);
+    return PyUnicode_FromString (ad->gecos);
   else if (strcmp (name, "dir") == 0)
-    return PyString_FromString (ad->dir);
+    return PyUnicode_FromString (ad->dir);
   else if (strcmp (name, "shell") == 0)
-    return PyString_FromString (ad->shell);
+    return PyUnicode_FromString (ad->shell);
   else if (strcmp (name, "mailbox") == 0)
-    return PyString_FromString (ad->mailbox);
+    return PyUnicode_FromString (ad->mailbox);
   else if (strcmp (name, "source") == 0)
-    return PyString_FromString (ad->source);
+    return PyUnicode_FromString (ad->source);
   else if (strcmp (name, "quota") == 0)
     /* FIXME: quota is mu_off_t rather than size_t.  See comment for uid
        above */
-    return PyInt_FromSize_t (ad->quota);
+    return PyLong_FromSize_t (ad->quota);
   else if (strcmp (name, "flags") == 0)
-    return PyInt_FromLong (ad->flags);
+    return PyLong_FromLong (ad->flags);
   else if (strcmp (name, "change_uid") == 0)
-    return PyInt_FromLong (ad->change_uid);
+    return PyLong_FromLong (ad->change_uid);
   return NULL;
 }
 
 static PyTypeObject PyAuthDataType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME4,  /* tp_name */
-  sizeof (PyAuthData),       /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_dealloc4,     /* tp_dealloc */
-  0,                         /* tp_print */
-  _getattr4,                 /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr4,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr4,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME4,
+  .tp_basicsize = sizeof (PyAuthData),
+  .tp_dealloc = (destructor)_dealloc4,
+  .tp_getattr = _getattr4,
+  .tp_repr = _repr4,
+  .tp_str = _repr4,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PyAuthority *
@@ -341,7 +218,7 @@ api_authority_create (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_authority_create (&py_auth->auth, py_ticket->ticket, NULL);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -385,7 +262,7 @@ api_authority_set_ticket (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_authority_set_ticket (py_auth->auth, py_ticket->ticket);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -398,7 +275,7 @@ api_authority_authenticate (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_authority_authenticate (py_auth->auth);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 /*
@@ -415,7 +292,7 @@ api_ticket_create (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_ticket_create (&py_ticket->ticket, NULL);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -448,7 +325,7 @@ api_ticket_set_secret (PyObject *self, PyObject *args)
 
   status = mu_ticket_set_secret (py_ticket->ticket,
 				 py_secret->secret);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 /*
@@ -466,7 +343,7 @@ api_wicket_create (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_file_wicket_create (&py_wicket->wicket, filename);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -560,7 +437,7 @@ api_register_module (PyObject *self, PyObject *args)
     return NULL;
 
   status = register_module (name);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -611,7 +488,7 @@ api_authenticate (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_authenticate (py_ad->auth_data, pass);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -678,6 +555,14 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
 int
 mu_py_init_auth (void)
 {
@@ -701,7 +586,7 @@ void
 _mu_py_attach_auth (void)
 {
   PyObject *m;
-  if ((m = _mu_py_attach_module (PY_MODULE, methods)))
+  if ((m = _mu_py_attach_module (&moduledef)))
     {
       Py_INCREF (&PyAuthorityType);
       Py_INCREF (&PyTicketType);
